import 'dart:async';
import 'dart:io';

import 'package:args/command_runner.dart';
import 'package:console/console.dart';
import 'package:polyglothq/src/common/util/command_line_utils.dart';
import 'package:polyglothq/src/domain/commands/command_login.dart';
import 'package:polyglothq/src/domain/repository/api_repository.dart';
import 'package:polyglothq/src/domain/repository/token_repository.dart';

class CommandInit extends Command {
  CommandInit(this._commandLogin, this._apiRepository, this._tokenRepository);

  final CommandLogin _commandLogin;
  final ApiRepository _apiRepository;
  final TokenRepository _tokenRepository;

  @override
  String get name => 'init';

  @override
  String get description =>
      'Sets up a polyglot.yml config for selected project.';

  @override
  FutureOr<void> run() async {
    while (!await _tokenRepository.loadToken()) {
      await _commandLogin.run();
    }

    stdout.writeln('Getting projects...');
    try {
      final projects = await _apiRepository.getProjects();
      projects.sort((a, b) => a.name.compareTo(b.name));

      stdout.writeln('Choose a project: ');
      stdout.writeln('  ID  => Name:');
      stdout.writeln('  ------------');
      stdout.writeln('');
      for (final project in projects) {
        stdout.writeln(
            '  ${project.id.toString().padRight(3)} => ${project.name}');
      }

      stdout.write('Enter project ID: ');
      final id = stdin.readLineSync();

      final flutterSdk = await CommandLineUtils.readInputWithDefault(
          'What Flutter instance are you using (flutter/fvm flutter/...)?',
          'flutter');
      final storageFile = await CommandLineUtils.readInputWithDefault(
          'Where would you like to store translation files (.arb)?',
          './assets/i18n/');
      final sourceFiles = await CommandLineUtils.readInputWithDefault(
          'Where would you like to store source files?',
          './lib/util/localization/');
      final mainLanguage = await CommandLineUtils.readInputWithDefault(
          'What is the mandatory (main) language on polyglot?', 'en');

      final File y = File('polyglot.yml');
      final content = '''---
:projects:
  - :id: $id
    :flutterInstance: $flutterSdk
    :mandatoryLanguage: $mainLanguage
    :path: $storageFile
    :l10nFilePath: ./l10n.yaml
    :sourceFilesPath: $sourceFiles
    :outputLocalizationFile: app_localizations
    :setDefaultLanguage: false
    :defaultCountryCode:
  #    en: GB
''';

      y.writeAsStringSync(content, mode: FileMode.writeOnly);
      stdout.writeln('Success!');
      exit(0);
    } catch (e, st) {
      print(e);
      print(st);
      TextPen().red().text('Error fetching projects').print();
      exit(2);
    }
  }
}
