import 'dart:async';
import 'dart:io';

import 'package:args/command_runner.dart';
import 'package:polyglothq/src/common/model/polyglot_content.dart';
import 'package:polyglothq/src/domain/arb_mapper.dart';
import 'package:polyglothq/src/domain/commands/command_login.dart';
import 'package:polyglothq/src/domain/repository/api_repository.dart';
import 'package:polyglothq/src/domain/repository/token_repository.dart';
import 'package:yaml/yaml.dart';

class CommandPull extends Command {
  CommandPull(
    this._commandLogin,
    this._apiRepository,
    this._arbMapper,
    this._tokenRepository,
  );

  final CommandLogin _commandLogin;
  final ApiRepository _apiRepository;
  final TokenRepository _tokenRepository;
  final ArbMapper _arbMapper;

  @override
  String get name => 'pull';

  @override
  String get description =>
      'Pulls all translations from Polyglot and generates necessary files';

  @override
  FutureOr<void> run() async {
    while (!await _tokenRepository.loadToken()) {
      await _commandLogin.run();
    }

    final File polyglot = File('polyglot.yml');
    if (!await polyglot.exists()) {
      stderr.writeln('We could not find a file that we need:');
      stderr.writeln('No such file or directory @ - ${polyglot.absolute.path}');
    }

    final doc = loadYaml(polyglot.readAsStringSync());
    final polyglotContent = PolyglotContent.fromYaml(doc);
    final langs = await _apiRepository.getLanguages(polyglotContent.projectId);
    final Set<String> locales = {};
    for (final lang in langs) {
      final String langCode = lang.locale.split('_').first;
      final String countryCode = lang.locale.split('_')[1];
      final String localeName =
          polyglotContent.defaultCountryCode[langCode] == null
              ? lang.locale
              : polyglotContent.defaultCountryCode[langCode] == countryCode
                  ? langCode
                  : lang.locale;
      // late final String localeName;
      // if (locales.contains(lang.locale.split('_').first)) {
      //   localeName = lang.locale;
      // } else {
      //   localeName = lang.locale.split('_').first;
      // }
      locales.add(localeName);
      final apiJson = await _apiRepository.getTranslations(
          polyglotContent.projectId, lang.id);
      final arbJson = _arbMapper.map(apiJson, localeName);
      final file = await File('${polyglotContent.path}intl_$localeName.arb')
          .create(recursive: true);
      final openedFile = file.openSync(mode: FileMode.write);
      openedFile.writeStringSync(arbJson);
      openedFile.closeSync();
    }

    final file2 =
        await File(polyglotContent.l10nFilePath).create(recursive: true);
    final openedFile2 = file2.openSync(mode: FileMode.write);
    openedFile2.writeStringSync('''---
arb-dir: "${polyglotContent.path}"
template-arb-file: intl_${polyglotContent.mandatoryLanguage}.arb
output-localization-file: ${polyglotContent.outputLocalizationFile}.dart
''');
    openedFile2.closeSync();

    final directory = await Directory(polyglotContent.sourceFilesPath)
        .create(recursive: true);
    final file3 =
        await File('${directory.path}/strings.dart').create(recursive: true);
    final openedFile3 = file3.openSync(mode: FileMode.write);
    openedFile3.writeStringSync('''import 'package:flutter/material.dart';
import 'package:flutter_gen/gen_l10n/app_localizations.dart';
${_defaultLanguageImport(polyglotContent)}
class Strings {
  Strings._();

${_latestLocalizationsDeclaration(polyglotContent)}

  static AppLocalizations of(BuildContext context) => AppLocalizations.of(context) ?? latest;

  static AppLocalizations get latest {
    final appLocalizations = _latestLocalizations;
    if (appLocalizations == null) {
      throw 'Unable to load App Localizations';
    }
    return appLocalizations;
  }
}

class StringsLocaleWidget extends StatelessWidget {
  const StringsLocaleWidget({Key? key, required this.child}) : super(key: key);
  final Widget child;

  @override
  Widget build(BuildContext context) {
    Strings._latestLocalizations = AppLocalizations.of(context);
    return child;
  }
}

${_getLocales(locales)}
''');
    openedFile3.closeSync();

    final commands = '${polyglotContent.flutterInstance} pub get'.split(' ');
    final out = await Process.run(commands.first, commands.sublist(1));
    stdout.writeln(out.stderr);
    stdout.writeln('Completed fetching');
  }

  String _latestLocalizationsDeclaration(PolyglotContent polyglotContent) {
    if (polyglotContent.setDefaultLanguage) {
      return '  static AppLocalizations? _latestLocalizations = AppLocalizations${capitalizeFirstLetter(polyglotContent.mandatoryLanguage)}();';
    }
    return '  static AppLocalizations? _latestLocalizations;';
  }

  String _defaultLanguageImport(PolyglotContent polyglotContent) {
    if (polyglotContent.setDefaultLanguage) {
      return 'import \'package:flutter_gen/gen_l10n/app_localizations_${polyglotContent.mandatoryLanguage}.dart\';\n';
    }
    return '';
  }

  String _getLocales(Set<String> locales) {
    StringBuffer sb = StringBuffer();
    final Set<String> localeNames = {};
    sb.writeln('extension AppLocales on Locale {');
    for (var element in locales) {
      final splitted = element.split('_');
      final languageCode = splitted.first;
      String? countryCode;
      if (splitted.length == 2) {
        countryCode = splitted[1];
      }
      final localeName =
          '${capitalizeFirstLetter(languageCode)}${capitalizeFirstLetter(countryCode) ?? ''}';
      localeNames.add(localeName);
      sb.writeln(
          "  static Locale get locale$localeName => const Locale('$languageCode', '${countryCode ?? ''}');");
      sb.writeln();
    }
    sb.writeln('  static List<Locale> get values => <Locale>[');
    for (var locale in localeNames) {
      sb.writeln('        AppLocales.locale$locale,');
    }
    sb.writeln('      ];');
    sb.write('}');
    return sb.toString();
  }

  String? capitalizeFirstLetter(String? string) {
    if (string == null) return null;
    return '${string[0].toUpperCase()}${string.substring(1)}';
  }
}
